import * as fs from 'fs';
import * as ejs from 'ejs';
import * as path from 'path';
import * as mysql from 'mysql2';
import chalk from 'chalk';
import moment from 'moment';
import ora from 'ora';
import inquirer from 'inquirer';
import { fileURLToPath } from 'url';
import { hump, file } from './common/transitionTableName.mjs';
import { config as templatePighandSpring } from './template/config/pighand-spring.mjs';
import { config as templatePighandSpringV1 } from './template/config/pighand-spring-v1.mjs';
import { config as templatePighandKoa } from './template/config/pighand-koa.mjs';
import { config as templatePighandDashboardVue } from './template/config/pighand-dashboard-vue.mjs';
import { getTableComment, getTableColumn, getAllTables, getTableRelation } from './common/mysql.mjs';
import { templateType, inquirerConflict, } from './common/config.mjs';
class Auto {
    config;
    spinner;
    // 是否批量操作
    isBatch = false;
    constructor(options, spinner) {
        this.spinner = spinner;
        const { tableName, params, paramsPath, saveFileRootPath } = options;
        // 过滤空表名
        const tableNamesFormat = (tableName && Array.from(new Set(tableName.filter((item) => !!item.trim())))) || [];
        // 根据java关键字，处理默认java package
        const paths = path.resolve(saveFileRootPath).split(path.sep);
        const javaPathIndex = paths.indexOf('java') + 1;
        const javaPackage = paths.slice(javaPathIndex).join('.');
        // 处理自定义参数
        let customParams = {
            javaPackage,
            dateTime: moment().format('YYYY-MM-DD HH:mm:ss'),
            author: process.env.USER,
        };
        // 自定义参数
        (params || '').split('&').forEach((item) => {
            const paramsItems = item.split('=');
            if (paramsItems.length >= 2) {
                const [paramsItemFirst, ...paramsItemsOther] = paramsItems;
                customParams[paramsItemFirst] = paramsItemsOther.join('=');
            }
        });
        // 自定义参数文件
        if (paramsPath) {
            const paramsFile = fs.readFileSync(paramsPath).toString();
            const paramsFileJson = JSON.parse(paramsFile);
            customParams = {
                ...customParams,
                ...paramsFileJson,
            };
        }
        this.config = {
            ...options,
            tableNames: tableNamesFormat,
            customParams,
        };
    }
    async run() {
        const { customParams } = this.config;
        const { tables, tableColumnMap, tableRelationMap } = await this.getTableInfos();
        this.isBatch = false;
        for (const table of tables) {
            const { tableName, tableComment } = table;
            const params = {
                ...customParams,
                tableName,
                tableComment,
                tableFileName: file(tableName),
                tableHumpName: hump(tableName),
                columns: tableColumnMap.get(table.tableName),
                tableRelationMap,
            };
            this.spinner.text = chalk.blue(tableName);
            // 生成文件
            await this.generateFile(params);
        }
    }
    /**
     * 获取表信息
     * @returns {Array<TableInfo>} tables 表备注信息
     * @returns {Map{表名: ColumnInfo}} tableColumnMap 表列信息
     */
    async getTableInfos() {
        let tables = [];
        let tableColumns = [];
        const { host, port, user, password, database, tableNames } = this.config;
        // 连接数据库
        const pool = mysql.createConnection({
            host,
            port,
            user,
            password,
            database,
        });
        const connection = pool.promise();
        // 查询表信息
        if (tableNames && tableNames.length) {
            // 指定表明
            tables = await getTableComment(connection, database, tableNames);
            tableColumns = await getTableColumn(connection, database, tableNames);
        }
        else {
            // 全部表
            tables = await getAllTables(connection, database);
            const allTableNames = tables.map((item) => item.tableName);
            tableColumns = await getTableColumn(connection, database, allTableNames);
        }
        // 查询表关联信息
        const tableRelationMap = await getTableRelation(connection, database);
        await connection.end();
        // 根据表明格式化列
        // {表名: ColumnInfo}
        const tableColumnMap = new Map();
        tableColumns.forEach((item) => {
            const { tableName } = item;
            const columnInfos = tableColumnMap.get(tableName) || [];
            columnInfos.push(item);
            tableColumnMap.set(tableName, columnInfos);
        });
        return {
            tables,
            tableColumnMap,
            tableRelationMap,
        };
    }
    /**
     * 生成文件
     * @param params
     */
    async generateFile(params) {
        const { template, templatePath, saveFileRootPath } = this.config;
        // 模板类型
        // customize 自定义，使用绝对路径查找配置文件
        // internal 内置，使用相对路径查找配置文件
        let templatePathType = 'customize';
        // 读取模板配置
        let templateConfig = [];
        if (templatePath) {
            // 根据配置文件路径读取模板配置
            const templateConfigFileBuffer = fs.readFileSync(templatePath);
            templateConfig = JSON.parse(templateConfigFileBuffer.toString());
        }
        else {
            // 使用内置模板
            templatePathType = 'internal';
            switch (template) {
                case templateType.pighandSpring:
                    templateConfig = templatePighandSpring;
                    break;
                case templateType.pighandSpringV1:
                    templateConfig = templatePighandSpringV1;
                    break;
                case templateType.pighandKoa:
                    templateConfig = templatePighandKoa;
                    break;
                case templateType.pighandDashboardVue:
                    templateConfig = templatePighandDashboardVue;
                    break;
            }
        }
        const __filename = fileURLToPath(import.meta.url);
        const __dirname = path.dirname(__filename);
        for (const item of templateConfig) {
            const { name, templateFile, saveFilePath = name, fileExtension } = item;
            // 路径：filePath（不存在默认使用name）/文件名 + 扩展名
            const filePath = path.join(saveFileRootPath || __dirname, saveFilePath);
            const fileName = `${params.tableFileName}${file(name)}.${fileExtension}`;
            // 目录是否存在
            const isPathExist = fs.existsSync(filePath);
            if (!isPathExist) {
                fs.mkdirSync(filePath, { recursive: true });
            }
            const saveFile = path.join(filePath, fileName);
            const isFileExist = fs.existsSync(saveFile);
            // 使用模板生成文件
            let newFileString = '';
            try {
                const templateFilePath = templatePathType === 'internal' ? path.resolve(__dirname, templateFile) : templateFile;
                const templateFileString = fs.readFileSync(templateFilePath).toString();
                newFileString = ejs.render(templateFileString, params);
            }
            catch (e) {
                ora(chalk.redBright(path.resolve(saveFile))).fail();
                throw e;
            }
            // 如果文件存在，且未执行批量操作，提示用户选择操作方式
            if (isFileExist && !this.isBatch) {
                // 显示冲突文件
                this.spinner.warn(chalk.yellow(saveFile));
                // 输入参数不正确，重新提示选择
                let conflict;
                while (!conflict) {
                    const prompt = inquirer.createPromptModule();
                    const op = await prompt(inquirerConflict);
                    if (inquirerConflict[0].choices.includes(op.conflict)) {
                        conflict = op.conflict;
                    }
                }
                if (conflict === 'n') {
                    continue;
                }
                else if (conflict === 'N') {
                    break;
                }
                else if (conflict === 'R') {
                    this.isBatch = true;
                }
            }
            fs.writeFileSync(saveFile, newFileString);
            ora(chalk.green(path.resolve(saveFile))).succeed();
        }
    }
}
export default Auto;
